"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.authConnect = exports.Else = exports.Then = exports.Access = exports.IfNotGranted = exports.Username = exports.configureAccessDecisionManager = exports.ANONYMOUS = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _reactIf = require("react-if");

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require("lodash.isempty");

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CONTEXT_KEY = 'react-if-auth';
var ANONYMOUS = exports.ANONYMOUS = '__ANONYMOUS__';

var NO_ROLES = [];
var DEFAULT_ROLE_NAME_PREFIX = 'ROLE_';
var ANONYMOUS_AUTHENTICATION = {
    username: ANONYMOUS,
    roles: NO_ROLES
};

function createProvider() {
    var contextKey = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : CONTEXT_KEY;

    var AuthProvider = function (_React$Component) {
        _inherits(AuthProvider, _React$Component);

        _createClass(AuthProvider, [{
            key: "getChildContext",
            value: function getChildContext() {
                return _defineProperty({}, contextKey, this[contextKey]);
            }
        }]);

        function AuthProvider(props, context) {
            _classCallCheck(this, AuthProvider);

            var _this = _possibleConstructorReturn(this, (AuthProvider.__proto__ || Object.getPrototypeOf(AuthProvider)).call(this, props, context));

            _this[contextKey] = props.accessDecisionManager;
            return _this;
        }

        _createClass(AuthProvider, [{
            key: "render",
            value: function render() {
                return _react.Children.only(this.props.children);
            }
        }]);

        return AuthProvider;
    }(_react2.default.Component);

    AuthProvider.propTypes = {
        accessDecisionManager: _propTypes2.default.object.isRequired,
        children: _propTypes2.default.element.isRequired
    };
    AuthProvider.childContextTypes = _defineProperty({}, contextKey, _propTypes2.default.object.isRequired);
    return AuthProvider;
}

var connect = function connect(ComponentToWrap) {
    var _class, _temp;

    var contextKey = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : CONTEXT_KEY;
    var AuthComponent = (_temp = _class = function (_Component) {
        _inherits(AuthComponent, _Component);

        function AuthComponent() {
            _classCallCheck(this, AuthComponent);

            return _possibleConstructorReturn(this, (AuthComponent.__proto__ || Object.getPrototypeOf(AuthComponent)).apply(this, arguments));
        }

        _createClass(AuthComponent, [{
            key: "render",
            value: function render() {
                var accessDecisionManager = this.context[contextKey];

                return _react2.default.createElement(ComponentToWrap, _extends({}, this.props, { accessDecisionManager: accessDecisionManager }));
            }
            // let’s define what’s needed from the `context`

        }]);

        return AuthComponent;
    }(_react.Component), _class.contextTypes = _defineProperty({}, contextKey, _propTypes2.default.object.isRequired), _temp);

    return AuthComponent;
};

var defaults = {
    principalResolver: function principalResolver(auth) {
        return auth;
    },
    rolesResolver: function rolesResolver(auth) {
        return auth && auth.roles ? auth.roles : NO_ROLES;
    },
    usernameResolver: function usernameResolver(auth) {
        return auth ? auth.username : null;
    },
    rememberMeResolver: function rememberMeResolver(auth) {
        return auth ? auth.remembered || false : false;
    },
    roleNameResolver: function roleNameResolver(role, prefix) {
        return role.startsWith(prefix) ? role : prefix + role;
    },
    isAuthenticatedResolver: function isAuthenticatedResolver(authentication) {
        return !(0, _lodash2.default)(authentication);
    },
    defaultRolePrefix: DEFAULT_ROLE_NAME_PREFIX,
    anonymous: ANONYMOUS_AUTHENTICATION
};

function configureAccessDecisionManager(args) {
    var _defaults$args = _extends({}, defaults, args),
        authLookup = _defaults$args.authLookup,
        isAuthenticatedResolver = _defaults$args.isAuthenticatedResolver,
        principalResolver = _defaults$args.principalResolver,
        rolesResolver = _defaults$args.rolesResolver,
        usernameResolver = _defaults$args.usernameResolver,
        rememberMeResolver = _defaults$args.rememberMeResolver,
        roleNameResolver = _defaults$args.roleNameResolver,
        defaultRolePrefix = _defaults$args.defaultRolePrefix,
        anonymous = _defaults$args.anonymous;

    if (!authLookup || typeof authLookup !== 'function') {
        throw new Error('Function authLookup is required and is not specified');
    }

    var AccessDecisionManager = function () {
        function AccessDecisionManager() {
            _classCallCheck(this, AccessDecisionManager);
        }

        _createClass(AccessDecisionManager, [{
            key: "decide",
            value: function decide(expression, authentication) {
                return expression(this.createEvaluator(authentication));
            }
        }, {
            key: "getAuthentication",
            value: function getAuthentication() {
                var authentication = authLookup();
                if (!authentication) {
                    authentication = anonymous;
                }
                return authentication;
            }
        }, {
            key: "getPrincipal",
            value: function getPrincipal(authentication) {
                return principalResolver(authentication);
            }
        }, {
            key: "isAuthenticated",
            value: function isAuthenticated(authentication) {
                return isAuthenticatedResolver(authentication);
            }
        }, {
            key: "isRememberMe",
            value: function isRememberMe(authentication) {
                return rememberMeResolver(authentication);
            }
        }, {
            key: "isAnonymous",
            value: function isAnonymous(authentication) {
                return ANONYMOUS === this.getUsername(authentication);
            }
        }, {
            key: "getUsername",
            value: function getUsername(authentication) {
                return usernameResolver(authentication);
            }
        }, {
            key: "getRoles",
            value: function getRoles(authentication) {
                return rolesResolver(authentication);
            }
        }, {
            key: "getRole",
            value: function getRole(role) {
                return roleNameResolver(role, defaultRolePrefix);
            }
        }, {
            key: "createEvaluator",
            value: function createEvaluator(authentication) {
                return new SpelExpressionEvaluator(this, authentication || this.getAuthentication());
            }
        }]);

        return AccessDecisionManager;
    }();

    return new AccessDecisionManager();
}

var SpelExpressionEvaluator = function () {
    function SpelExpressionEvaluator(accessDecisionManager, authentication) {
        _classCallCheck(this, SpelExpressionEvaluator);

        this.authentication = authentication;
        this.accessDecisionManager = accessDecisionManager;
        this.roles = this.anonymous = this.rememberMe = this.username = this.principal = this.authenticated = null;
    }

    _createClass(SpelExpressionEvaluator, [{
        key: "getPrincipal",
        value: function getPrincipal() {
            if (this.principal === null) this.principal = this.accessDecisionManager.getPrincipal(this.authentication);
            return this.principal;
        }
    }, {
        key: "hasAuthority",
        value: function hasAuthority(authority) {
            if (this.isAuthenticated()) {
                var hasAuthority = this.getRoles().indexOf(authority) !== -1;
                return hasAuthority;
            }
            return false;
        }
    }, {
        key: "getRoles",
        value: function getRoles() {
            if (this.roles === null) this.roles = this.accessDecisionManager.getRoles(this.authentication);
            return this.roles;
        }
    }, {
        key: "getUsername",
        value: function getUsername() {
            if (this.username === null) this.username = this.accessDecisionManager.getUsername(this.authentication);
            return this.username;
        }
    }, {
        key: "hasAnyAuthority",
        value: function hasAnyAuthority(authorities) {
            authorities = this.ensureArray(authorities);

            for (var i = 0; i < authorities.length; i++) {
                var authority = authorities[i];
                if (this.hasAuthority(authority)) return true;
            }
            return false;
        }
    }, {
        key: "isAnonymous",
        value: function isAnonymous() {
            if (this.anonymous === null) {
                this.anonymous = this.accessDecisionManager.isAnonymous(this.authentication);
            }
            return this.anonymous;
        }
    }, {
        key: "isRememberMe",
        value: function isRememberMe() {
            if (this.rememberMe === null) {
                this.rememberMe = this.accessDecisionManager.isRememberMe(this.authentication);
            }
            return this.rememberMe;
        }
    }, {
        key: "isAuthenticated",
        value: function isAuthenticated() {
            if (this.authenticated === null) {
                this.authenticated = this.accessDecisionManager.isAuthenticated(this.authentication);
            }
            return this.authenticated && !this.isAnonymous();
        }
    }, {
        key: "isFullyAuthenticated",
        value: function isFullyAuthenticated() {
            return this.isAuthenticated() && !this.isRememberMe();
        }
    }, {
        key: "hasRole",
        value: function hasRole(role) {
            return this.hasAuthority(this.getRole(role));
        }
    }, {
        key: "permitAll",
        value: function permitAll() {
            return true;
        }
    }, {
        key: "denyAll",
        value: function denyAll() {
            return false;
        }
    }, {
        key: "hasAnyRole",
        value: function hasAnyRole(roles) {
            roles = this.ensureArray(roles);

            for (var i = 0; i < roles.length; i++) {
                var role = roles[i];
                if (this.hasRole(role)) return true;
            }
            return false;
        }
    }, {
        key: "getRole",
        value: function getRole(role) {
            return this.accessDecisionManager.getRole(role);
        }
    }, {
        key: "ensureArray",
        value: function ensureArray(value) {
            if (typeof value === 'string') {
                if (value.contains(',')) {
                    return value.split(',');
                } else {
                    return [value];
                }
            }
            ;
            if (Array.isArray(value)) return value;
            throw new Error('Request parameter is not an Array');
        }
    }]);

    return SpelExpressionEvaluator;
}();

var Username = connect(function (props) {
    var securityConfig = props.securityConfig;

    return _react2.default.createElement(
        "span",
        null,
        securityConfig.getAuthentication().username
    );
});

var IfNotGranted = connect(function (props) {
    var roles = props.roles,
        rest = _objectWithoutProperties(props, ["roles"]);

    return _react2.default.createElement(Access, _extends({}, rest, { expression: function expression(sec) {
            return !sec.hasAnyRole(roles);
        } }));
});
var Access = connect(function (props) {
    var accessDecisionManager = props.accessDecisionManager,
        expression = props.expression,
        children = props.children,
        rest = _objectWithoutProperties(props, ["accessDecisionManager", "expression", "children"]);

    var condition = function condition() {
        return accessDecisionManager.decide(expression);
    };
    return _react2.default.createElement(
        _reactIf.If,
        _extends({}, rest, { condition: condition() }),
        children
    );
});

exports.default = createProvider();
exports.configureAccessDecisionManager = configureAccessDecisionManager;
exports.Username = Username;
exports.IfNotGranted = IfNotGranted;
exports.Access = Access;
exports.Then = _reactIf.Then;
exports.Else = _reactIf.Else;
exports.authConnect = connect;